module Functions.Text
  ( concatStrings
  , concatTexts
  , lowerString
  , toText
  , readTextFile
  , saveTextFile
  ) where

import Types (IsFunctionIO(..), Token)
import DependencyGenerator (DepGenM, TokenableTo(..), onToken, toTupleToken,
                            runFunctionIO, runFunctionIO_)

import Data.Char (toLower)
import Data.Text (Text)
import qualified Data.Text as T
import qualified Data.Text.IO as T

concatStrings :: TokenableTo [String] a => a -> DepGenM (Token String)
concatStrings = onToken concat

concatTexts :: TokenableTo [Text] a => a -> DepGenM (Token Text)
concatTexts = onToken T.concat

lowerString :: TokenableTo String a => a -> DepGenM (Token String)
lowerString = onToken (map toLower)

toText :: TokenableTo String a => a -> DepGenM (Token Text)
toText = onToken T.pack


data ReadTextFile = ReadTextFile deriving Show
instance IsFunctionIO ReadTextFile FilePath Text where
  evalFunctionIO ReadTextFile s =
    T.readFile s
  functionIOTouchesFilesystem ReadTextFile = False

readTextFile :: TokenableTo FilePath a => a -> DepGenM (Token Text)
readTextFile a = runFunctionIO ReadTextFile =<< toToken a


data SaveTextFile = SaveTextFile deriving Show
instance IsFunctionIO SaveTextFile (Text, FilePath) () where
  evalFunctionIO SaveTextFile (t, s) =
    T.writeFile s t
  functionIOTouchesFilesystem SaveTextFile = True

saveTextFile :: (TokenableTo Text a, TokenableTo FilePath b) => a -> b -> DepGenM ()
saveTextFile a b = runFunctionIO_ SaveTextFile =<< toTupleToken a b
